@@ -17,16 +17,17 @@ import sys
 from getopt import getopt, GetoptError
 
 from traitlets.config.configurable import Configurable
-from IPython.core import oinspect
-from IPython.core.error import UsageError
-from IPython.core.inputtransformer2 import ESC_MAGIC, ESC_MAGIC2
-from decorator import decorator
-from IPython.utils.ipstruct import Struct
-from IPython.utils.process import arg_split
-from IPython.utils.text import dedent
+from . import oinspect
+from .error import UsageError
+from .inputtransformer2 import ESC_MAGIC, ESC_MAGIC2
+from ..utils.ipstruct import Struct
+from ..utils.process import arg_split
+from ..utils.text import dedent
 from traitlets import Bool, Dict, Instance, observe
 from logging import error
 
+import typing as t
+
 #-----------------------------------------------------------------------------
 # Globals
 #-----------------------------------------------------------------------------
@@ -37,7 +38,7 @@ from logging import error
 # access to the class when they run.  See for more details:
 # http://stackoverflow.com/questions/2366713/can-a-python-decorator-of-an-instance-method-access-the-class
 
-magics = dict(line={}, cell={})
+magics: t.Dict = dict(line={}, cell={})
 
 magic_kinds = ('line', 'cell')
 magic_spec = ('line', 'cell', 'line_cell')
@@ -115,16 +116,13 @@ def record_magic(dct, magic_kind, magic_name, func):
     Parameters
     ----------
     dct : dict
-      A dictionary with 'line' and 'cell' subdicts.
-
+        A dictionary with 'line' and 'cell' subdicts.
     magic_kind : str
-      Kind of magic to be stored.
-
+        Kind of magic to be stored.
     magic_name : str
-      Key to store the magic as.
-
+        Key to store the magic as.
     func : function
-      Callable object to store.
+        Callable object to store.
     """
     if magic_kind == 'line_cell':
         dct['line'][magic_name] = dct['cell'][magic_name] = func
@@ -184,20 +182,18 @@ def _method_magic_marker(magic_kind):
     # This is a closure to capture the magic_kind.  We could also use a class,
     # but it's overkill for just that one bit of state.
     def magic_deco(arg):
-        call = lambda f, *a, **k: f(*a, **k)
-
         if callable(arg):
             # "Naked" decorator call (just @foo, no args)
             func = arg
             name = func.__name__
-            retval = decorator(call, func)
+            retval = arg
             record_magic(magics, magic_kind, name, name)
         elif isinstance(arg, str):
             # Decorator called with arguments (@foo('bar'))
             name = arg
             def mark(func, *a, **kw):
                 record_magic(magics, magic_kind, name, func.__name__)
-                return decorator(call, func)
+                return func
             retval = mark
         else:
             raise TypeError("Decorator can only be called with "
@@ -217,8 +213,6 @@ def _function_magic_marker(magic_kind):
     # This is a closure to capture the magic_kind.  We could also use a class,
     # but it's overkill for just that one bit of state.
     def magic_deco(arg):
-        call = lambda f, *a, **k: f(*a, **k)
-
         # Find get_ipython() in the caller's namespace
         caller = sys._getframe(1)
         for ns in ['f_locals', 'f_globals', 'f_builtins']:
@@ -236,13 +230,13 @@ def _function_magic_marker(magic_kind):
             func = arg
             name = func.__name__
             ip.register_magic_function(func, magic_kind, name)
-            retval = decorator(call, func)
+            retval = arg
         elif isinstance(arg, str):
             # Decorator called with arguments (@foo('bar'))
             name = arg
             def mark(func, *a, **kw):
                 ip.register_magic_function(func, magic_kind, name)
-                return decorator(call, func)
+                return func
             retval = mark
         else:
             raise TypeError("Decorator can only be called with "
@@ -265,7 +259,8 @@ def _function_magic_marker(magic_kind):
     return magic_deco
 
 
-MAGIC_NO_VAR_EXPAND_ATTR = '_ipython_magic_no_var_expand'
+MAGIC_NO_VAR_EXPAND_ATTR = "_ipython_magic_no_var_expand"
+MAGIC_OUTPUT_CAN_BE_SILENCED = "_ipython_magic_output_can_be_silenced"
 
 
 def no_var_expand(magic_func):
@@ -284,6 +279,16 @@ def no_var_expand(magic_func):
     return magic_func
 
 
+def output_can_be_silenced(magic_func):
+    """Mark a magic function so its output may be silenced.
+
+    The output is silenced if the Python code used as a parameter of
+    the magic ends in a semicolon, not counting a Python comment that can
+    follow it.
+    """
+    setattr(magic_func, MAGIC_OUTPUT_CAN_BE_SILENCED, True)
+    return magic_func
+
 # Create the actual decorators for public use
 
 # These three are used to decorate methods in class definitions
@@ -310,6 +315,34 @@ class MagicsManager(Configurable):
     # holding the actual callable object as value.  This is the dict used for
     # magic function dispatch
     magics = Dict()
+    lazy_magics = Dict(
+        help="""
+    Mapping from magic names to modules to load.
+
+    This can be used in IPython/IPykernel configuration to declare lazy magics
+    that will only be imported/registered on first use.
+
+    For example::
+
+        c.MagicsManager.lazy_magics = {
+          "my_magic": "slow.to.import",
+          "my_other_magic": "also.slow",
+        }
+
+    On first invocation of `%my_magic`, `%%my_magic`, `%%my_other_magic` or
+    `%%my_other_magic`, the corresponding module will be loaded as an ipython
+    extensions as if you had previously done `%load_ext ipython`.
+
+    Magics names should be without percent(s) as magics can be both cell
+    and line magics.
+
+    Lazy loading happen relatively late in execution process, and
+    complex extensions that manipulate Python/IPython internal state or global state
+    might not support lazy loading.
+    """
+    ).tag(
+        config=True,
+    )
 
     # A registry of the original objects that we've been given holding magics.
     registry = Dict()
@@ -374,10 +407,28 @@ class MagicsManager(Configurable):
             docs[m_type] = m_docs
         return docs
 
+    def register_lazy(self, name: str, fully_qualified_name: str):
+        """
+        Lazily register a magic via an extension.
+
+
+        Parameters
+        ----------
+        name : str
+            Name of the magic you wish to register.
+        fully_qualified_name :
+            Fully qualified name of the module/submodule that should be loaded
+            as an extensions when the magic is first called.
+            It is assumed that loading this extensions will register the given
+            magic.
+        """
+
+        self.lazy_magics[name] = fully_qualified_name
+
     def register(self, *magic_objects):
         """Register one or more instances of Magics.
 
-        Take one or more classes or instances of classes that subclass the main 
+        Take one or more classes or instances of classes that subclass the main
         `core.Magic` class, and register them with IPython to use the magic
         functions they provide.  The registration process will then ensure that
         any methods that have decorated to provide line and/or cell magics will
@@ -392,7 +443,7 @@ class MagicsManager(Configurable):
 
         Parameters
         ----------
-        magic_objects : one or more classes or instances
+        *magic_objects : one or more classes or instances
         """
         # Start by validating them to ensure they have all had their magic
         # methods registered at the instance level
@@ -415,7 +466,7 @@ class MagicsManager(Configurable):
 
         This will create an IPython magic (line, cell or both) from a
         standalone function.  The functions should have the following
-        signatures: 
+        signatures:
 
         * For line magics: `def f(line)`
         * For cell magics: `def f(line, cell)`
@@ -427,14 +478,12 @@ class MagicsManager(Configurable):
         Parameters
         ----------
         func : callable
-          Function to be registered as a magic.
-
+            Function to be registered as a magic.
         magic_kind : str
-          Kind of magic, one of 'line', 'cell' or 'line_cell'
-
+            Kind of magic, one of 'line', 'cell' or 'line_cell'
         magic_name : optional str
-          If given, the name the magic will have in the IPython namespace.  By
-          default, the name of the function itself is used.
+            If given, the name the magic will have in the IPython namespace.  By
+            default, the name of the function itself is used.
         """
 
         # Create the new method in the user_magics and register it in the
@@ -455,13 +504,11 @@ class MagicsManager(Configurable):
         Parameters
         ----------
         alias_name : str
-          The name of the magic to be registered.
-
+            The name of the magic to be registered.
         magic_name : str
-          The name of an existing magic.
-
+            The name of an existing magic.
         magic_kind : str
-          Kind of magic, one of 'line' or 'cell'
+            Kind of magic, one of 'line' or 'cell'
         """
 
         # `validate_type` is too permissive, as it allows 'line_cell'
@@ -585,25 +632,20 @@ class Magics(Configurable):
 
         Parameters
         ----------
-
         arg_str : str
-          The arguments to parse.
-
+            The arguments to parse.
         opt_str : str
-          The options specification.
-
+            The options specification.
         mode : str, default 'string'
-          If given as 'list', the argument string is returned as a list (split
-          on whitespace) instead of a string.
-
+            If given as 'list', the argument string is returned as a list (split
+            on whitespace) instead of a string.
         list_all : bool, default False
-          Put all option values in lists. Normally only options
-          appearing more than once are put in a list.
-
+            Put all option values in lists. Normally only options
+            appearing more than once are put in a list.
         posix : bool, default True
-          Whether to split the input line in POSIX mode or not, as per the
-          conventions outlined in the :mod:`shlex` module from the standard
-          library.
+            Whether to split the input line in POSIX mode or not, as per the
+            conventions outlined in the :mod:`shlex` module from the standard
+            library.
         """
 
         # inject default options at the beginning of the input line
@@ -618,6 +660,9 @@ class Magics(Configurable):
         posix = kw.get('posix', os.name == 'posix')
         strict = kw.get('strict', True)
 
+        preserve_non_opts = kw.get("preserve_non_opts", False)
+        remainder_arg_str = arg_str
+
         # Check if we have more than one argument to warrant extra processing:
         odict = {}  # Dictionary with options
         args = arg_str.split()
@@ -627,12 +672,21 @@ class Magics(Configurable):
             argv = arg_split(arg_str, posix, strict)
             # Do regular option processing
             try:
-                opts,args = getopt(argv, opt_str, long_opts)
+                opts, args = getopt(argv, opt_str, long_opts)
             except GetoptError as e:
-                raise UsageError('%s ( allowed: "%s" %s)' % (e.msg,opt_str,
-                                        " ".join(long_opts)))
-            for o,a in opts:
-                if o.startswith('--'):
+                raise UsageError(
+                    '%s (allowed: "%s"%s)'
+                    % (e.msg, opt_str, " ".join(("",) + long_opts) if long_opts else "")
+                ) from e
+            for o, a in opts:
+                if mode == "string" and preserve_non_opts:
+                    # remove option-parts from the original args-string and preserve remaining-part.
+                    # This relies on the arg_split(...) and getopt(...)'s impl spec, that the parsed options are
+                    # returned in the original order.
+                    remainder_arg_str = remainder_arg_str.replace(o, "", 1).replace(
+                        a, "", 1
+                    )
+                if o.startswith("--"):
                     o = o[2:]
                 else:
                     o = o[1:]
@@ -649,7 +703,10 @@ class Magics(Configurable):
         # Prepare opts,args for return
         opts = Struct(odict)
         if mode == 'string':
-            args = ' '.join(args)
+            if preserve_non_opts:
+                args = remainder_arg_str.lstrip()
+            else:
+                args = " ".join(args)
 
         return opts,args
 
@@ -661,7 +718,7 @@ class Magics(Configurable):
         self.options_table[fn] = optstr
 
 
-class MagicAlias(object):
+class MagicAlias:
     """An alias to another magic function.
 
     An alias is determined by its magic name and magic kind. Lookup
