@@ -9,23 +9,24 @@ from typing import Union as UnionType, Optional
 from IPython.core.async_helpers import get_asyncio_loop
 from IPython.core.interactiveshell import InteractiveShell, InteractiveShellABC
 from IPython.utils.py3compat import input
+from IPython.utils.PyColorize import theme_table
 from IPython.utils.terminal import toggle_set_term_title, set_term_title, restore_term_title
 from IPython.utils.process import abbrev_cwd
 from traitlets import (
+    Any,
     Bool,
-    Unicode,
     Dict,
+    Enum,
+    Float,
+    Instance,
     Integer,
     List,
-    observe,
-    Instance,
     Type,
-    default,
-    Enum,
+    Unicode,
     Union,
-    Any,
+    default,
+    observe,
     validate,
-    Float,
     DottedObjectName,
 )
 from traitlets.utils.importstring import import_item
@@ -42,11 +43,8 @@ from prompt_toolkit.patch_stdout import patch_stdout
 from prompt_toolkit.shortcuts import PromptSession, CompleteStyle, print_formatted_text
 from prompt_toolkit.styles import DynamicStyle, merge_styles
 from prompt_toolkit.styles.pygments import style_from_pygments_cls, style_from_pygments_dict
-from prompt_toolkit import __version__ as ptk_version
-
 from pygments.styles import get_style_by_name
 from pygments.style import Style
-from pygments.token import Token
 
 from .debugger import TerminalPdb, Pdb
 from .magics import TerminalMagics
@@ -67,39 +65,37 @@ from .shortcuts.auto_suggest import (
     AppendAutoSuggestionInAnyLine,
 )
 
-PTK3 = ptk_version.startswith('3.')
-
 
 class _NoStyle(Style):
     pass
 
 
-_style_overrides_light_bg = {
-            Token.Prompt: '#ansibrightblue',
-            Token.PromptNum: '#ansiblue bold',
-            Token.OutPrompt: '#ansibrightred',
-            Token.OutPromptNum: '#ansired bold',
-}
 
-_style_overrides_linux = {
-            Token.Prompt: '#ansibrightgreen',
-            Token.PromptNum: '#ansigreen bold',
-            Token.OutPrompt: '#ansibrightred',
-            Token.OutPromptNum: '#ansired bold',
-}
-
-
-def _backward_compat_continuation_prompt_tokens(method, width: int, *, lineno: int):
+def _backward_compat_continuation_prompt_tokens(
+    method, width: int, *, lineno: int, wrap_count: int
+):
     """
     Sagemath use custom prompt and we broke them in 8.19.
+
+    make sure to pass only width if method only support width
     """
     sig = inspect.signature(method)
+    extra = {}
+    params = inspect.signature(method).parameters
     if "lineno" in inspect.signature(method).parameters or any(
         [p.kind == p.VAR_KEYWORD for p in sig.parameters.values()]
     ):
-        return method(width, lineno=lineno)
-    else:
-        return method(width)
+        extra["lineno"] = lineno
+    if "line_number" in inspect.signature(method).parameters or any(
+        [p.kind == p.VAR_KEYWORD for p in sig.parameters.values()]
+    ):
+        extra["line_number"] = lineno
+
+    if "wrap_count" in inspect.signature(method).parameters or any(
+        [p.kind == p.VAR_KEYWORD for p in sig.parameters.values()]
+    ):
+        extra["wrap_count"] = wrap_count
+    return method(width, **extra)
 
 
 def get_default_editor():
@@ -207,13 +203,18 @@ class PtkHistoryAdapter(History):
 class TerminalInteractiveShell(InteractiveShell):
     mime_renderers = Dict().tag(config=True)
 
-    space_for_menu = Integer(6, help='Number of line at the bottom of the screen '
-                                     'to reserve for the tab completion menu, '
-                                     'search history, ...etc, the height of '
-                                     'these menus will at most this value. '
-                                     'Increase it is you prefer long and skinny '
-                                     'menus, decrease for short and wide.'
-                            ).tag(config=True)
+    min_elide = Integer(
+        30, help="minimum characters for filling with ellipsis in file completions"
+    ).tag(config=True)
+    space_for_menu = Integer(
+        6,
+        help="Number of line at the bottom of the screen "
+        "to reserve for the tab completion menu, "
+        "search history, ...etc, the height of "
+        "these menus will at most this value. "
+        "Increase it is you prefer long and skinny "
+        "menus, decrease for short and wide.",
+    ).tag(config=True)
 
     pt_app: UnionType[PromptSession, None] = None
     auto_suggest: UnionType[
@@ -303,9 +304,12 @@ class TerminalInteractiveShell(InteractiveShell):
 
     # We don't load the list of styles for the help string, because loading
     # Pygments plugins takes time and can cause unexpected errors.
-    highlighting_style = Union([Unicode('legacy'), Type(klass=Style)],
-        help="""The name or class of a Pygments style to use for syntax
-        highlighting. To see available styles, run `pygmentize -L styles`."""
+    highlighting_style = Union(
+        [Unicode("legacy"), Type(klass=Style)],
+        help="""Deprecated, and has not effect, use IPython themes
+
+        The name or class of a Pygments style to use for syntax
+        highlighting. To see available styles, run `pygmentize -L styles`.""",
     ).tag(config=True)
 
     @validate('editing_mode')
@@ -343,11 +347,17 @@ class TerminalInteractiveShell(InteractiveShell):
     @observe('highlighting_style')
     @observe('colors')
     def _highlighting_style_changed(self, change):
-        self.refresh_style()
+        assert change.new == change.new.lower()
+        if change.new != "legacy":
+            warn(
+                "highlighting_style is deprecated since 9.0 and have no effect, use themeing."
+            )
+            return
 
     def refresh_style(self):
-        self._style = self._make_style_from_name_or_cls(self.highlighting_style)
+        self._style = self._make_style_from_name_or_cls("legacy")
 
+    # TODO: deprecate this
     highlighting_style_overrides = Dict(
         help="Override highlighting format for specific tokens"
     ).tag(config=True)
@@ -425,6 +435,7 @@ class TerminalInteractiveShell(InteractiveShell):
         "Default is `'NavigableAutoSuggestFromHistory`'.",
         allow_none=True,
     ).tag(config=True)
+    _autosuggestions_provider: Any
 
     llm_constructor_kwargs = Dict(
         {},
@@ -434,6 +445,31 @@ class TerminalInteractiveShell(InteractiveShell):
         This is used to – for example – set the `model_id`""",
     ).tag(config=True)
 
+    llm_prefix_from_history = DottedObjectName(
+        "input_history",
+        help="""\
+    Fully Qualifed name of a function that takes an IPython history manager and
+    return a prefix to pass the llm provider in addition to the current buffer
+    text.
+
+    You can use:
+
+     - no_prefix
+     - input_history
+
+    As default value. `input_history` (default),  will use all the input history
+    of current IPython session
+
+    """,
+    ).tag(config=True)
+    _llm_prefix_from_history: Any
+
+    @observe("llm_prefix_from_history")
+    def _llm_prefix_from_history_changed(self, change):
+        name = change.new
+        self._llm_prefix_from_history = name
+        self._set_autosuggestions()
+
     llm_provider_class = DottedObjectName(
         None,
         allow_none=True,
@@ -448,24 +484,17 @@ class TerminalInteractiveShell(InteractiveShell):
         `stream_inline_completions`
     """,
     ).tag(config=True)
+    _llm_provider_class: Any = None
 
     @observe("llm_provider_class")
     def _llm_provider_class_changed(self, change):
         provider_class = change.new
-        if provider_class is not None:
-            warn(
-                "TerminalInteractiveShell.llm_provider_class is a provisional"
-                "  API as of IPython 8.32, and may change without warnings."
-            )
-            if isinstance(self.auto_suggest, NavigableAutoSuggestFromHistory):
-                self.auto_suggest._llm_provider = provider_class()
-            else:
-                self.log.warn(
-                    "llm_provider_class only has effects when using"
-                    "`NavigableAutoSuggestFromHistory` as auto_suggest."
-                )
+        self._llm_provider_class = provider_class
+        self._set_autosuggestions()
 
-    def _set_autosuggestions(self, provider):
+    def _set_autosuggestions(self, provider=None):
+        if provider is None:
+            provider = self.autosuggestions_provider
         # disconnect old handler
         if self.auto_suggest and isinstance(
             self.auto_suggest, NavigableAutoSuggestFromHistory
@@ -477,14 +506,34 @@ class TerminalInteractiveShell(InteractiveShell):
             self.auto_suggest = AutoSuggestFromHistory()
         elif provider == "NavigableAutoSuggestFromHistory":
             # LLM stuff are all Provisional in 8.32
-            if self.llm_provider_class:
-                llm_provider_constructor = import_item(self.llm_provider_class)
+            if self._llm_provider_class:
+                llm_provider_constructor = import_item(self._llm_provider_class)
                 llm_provider = llm_provider_constructor(**self.llm_constructor_kwargs)
             else:
                 llm_provider = None
             self.auto_suggest = NavigableAutoSuggestFromHistory()
             # Provisinal in 8.32
             self.auto_suggest._llm_provider = llm_provider
+
+            name = self.llm_prefix_from_history
+
+            if name == "no_prefix":
+
+                def no_prefix(history_manager):
+                    return ""
+
+                fun = no_prefix
+
+            elif name == "input_history":
+
+                def input_history(history_manager):
+                    return "\n".join([s[2] for s in history_manager.get_range()]) + "\n"
+
+                fun = input_history
+
+            else:
+                fun = import_item(name)
+            self.auto_suggest._llm_prefixer = fun
         else:
             raise ValueError("No valid provider.")
         if self.pt_app:
@@ -723,10 +772,13 @@ class TerminalInteractiveShell(InteractiveShell):
             def prompt():
                 prompt_text = "".join(x[1] for x in self.prompts.in_prompt_tokens())
                 lines = [input(prompt_text)]
-                prompt_continuation = "".join(x[1] for x in self.prompts.continuation_prompt_tokens())
-                while self.check_complete('\n'.join(lines))[0] == 'incomplete':
-                    lines.append( input(prompt_continuation) )
-                return '\n'.join(lines)
+                prompt_continuation = "".join(
+                    x[1] for x in self.prompts.continuation_prompt_tokens()
+                )
+                while self.check_complete("\n".join(lines))[0] == "incomplete":
+                    lines.append(input(prompt_continuation))
+                return "\n".join(lines)
+
             self.prompt_for_code = prompt
             return
 
@@ -736,8 +788,8 @@ class TerminalInteractiveShell(InteractiveShell):
         # Pre-populate history from IPython's history database
         history = PtkHistoryAdapter(self)
 
-        self._style = self._make_style_from_name_or_cls(self.highlighting_style)
-        self.style = DynamicStyle(lambda: self._style)
+        self.refresh_style()
+        ptk_s = DynamicStyle(lambda: self._style)
 
         editing_mode = getattr(EditingMode, self.editing_mode.upper())
 
@@ -749,7 +801,7 @@ class TerminalInteractiveShell(InteractiveShell):
             history=history,
             completer=IPythonPTCompleter(shell=self),
             enable_history_search=self.enable_history_search,
-            style=self.style,
+            style=ptk_s,
             include_default_pygments_style=False,
             mouse_support=self.mouse_support,
             enable_open_in_editor=self.extra_open_editor_shortcuts,
@@ -766,64 +818,28 @@ class TerminalInteractiveShell(InteractiveShell):
 
         We need that to add style for prompt ... etc.
         """
-        style_overrides = {}
-        if name_or_cls == 'legacy':
-            legacy = self.colors.lower()
-            if legacy == 'linux':
-                style_cls = get_style_by_name('monokai')
-                style_overrides = _style_overrides_linux
-            elif legacy == 'lightbg':
-                style_overrides = _style_overrides_light_bg
-                style_cls = get_style_by_name('pastie')
-            elif legacy == 'neutral':
-                # The default theme needs to be visible on both a dark background
-                # and a light background, because we can't tell what the terminal
-                # looks like. These tweaks to the default theme help with that.
-                style_cls = get_style_by_name('default')
-                style_overrides.update({
-                    Token.Number: '#ansigreen',
-                    Token.Operator: 'noinherit',
-                    Token.String: '#ansiyellow',
-                    Token.Name.Function: '#ansiblue',
-                    Token.Name.Class: 'bold #ansiblue',
-                    Token.Name.Namespace: 'bold #ansiblue',
-                    Token.Name.Variable.Magic: '#ansiblue',
-                    Token.Prompt: '#ansigreen',
-                    Token.PromptNum: '#ansibrightgreen bold',
-                    Token.OutPrompt: '#ansired',
-                    Token.OutPromptNum: '#ansibrightred bold',
-                })
+        assert name_or_cls == "legacy"
+        legacy = self.colors.lower()
 
-                # Hack: Due to limited color support on the Windows console
-                # the prompt colors will be wrong without this
-                if os.name == 'nt':
-                    style_overrides.update({
-                        Token.Prompt: '#ansidarkgreen',
-                        Token.PromptNum: '#ansigreen bold',
-                        Token.OutPrompt: '#ansidarkred',
-                        Token.OutPromptNum: '#ansired bold',
-                    })
-            elif legacy =='nocolor':
-                style_cls=_NoStyle
-                style_overrides = {}
-            else :
-                raise ValueError('Got unknown colors: ', legacy)
-        else :
-            if isinstance(name_or_cls, str):
-                style_cls = get_style_by_name(name_or_cls)
+        theme = theme_table.get(legacy, None)
+        assert theme is not None, legacy
+
+        if legacy == "nocolor":
+            style_overrides = {}
+            style_cls = _NoStyle
+        else:
+            style_overrides = {**theme.extra_style, **self.highlighting_style_overrides}
+            if theme.base is not None:
+                style_cls = get_style_by_name(theme.base)
             else:
-                style_cls = name_or_cls
-            style_overrides = {
-                Token.Prompt: '#ansigreen',
-                Token.PromptNum: '#ansibrightgreen bold',
-                Token.OutPrompt: '#ansired',
-                Token.OutPromptNum: '#ansibrightred bold',
-            }
-        style_overrides.update(self.highlighting_style_overrides)
-        style = merge_styles([
-            style_from_pygments_cls(style_cls),
-            style_from_pygments_dict(style_overrides),
-        ])
+                style_cls = _NoStyle
+
+        style = merge_styles(
+            [
+                style_from_pygments_cls(style_cls),
+                style_from_pygments_dict(style_overrides),
+            ]
+        )
 
         return style
 
@@ -839,6 +855,16 @@ class TerminalInteractiveShell(InteractiveShell):
     def color_depth(self):
         return (ColorDepth.TRUE_COLOR if self.true_color else None)
 
+    def _ptk_prompt_cont(self, width: int, line_number: int, wrap_count: int):
+        return PygmentsTokens(
+            _backward_compat_continuation_prompt_tokens(
+                self.prompts.continuation_prompt_tokens,
+                width,
+                lineno=line_number,
+                wrap_count=wrap_count,
+            )
+        )
+
     def _extra_prompt_options(self):
         """
         Return the current layout option for the current Terminal InteractiveShell
@@ -861,13 +887,7 @@ class TerminalInteractiveShell(InteractiveShell):
             "lexer": IPythonPTLexer(),
             "reserve_space_for_menu": self.space_for_menu,
             "message": get_message,
-            "prompt_continuation": (
-                lambda width, lineno, is_soft_wrap: PygmentsTokens(
-                    _backward_compat_continuation_prompt_tokens(
-                        self.prompts.continuation_prompt_tokens, width, lineno=lineno
-                    )
-                )
-            ),
+            "prompt_continuation": self._ptk_prompt_cont,
             "multiline": True,
             "complete_style": self.pt_complete_style,
             "input_processors": [
@@ -893,8 +913,6 @@ class TerminalInteractiveShell(InteractiveShell):
                 ),
             ],
         }
-        if not PTK3:
-            options['inputhook'] = self.inputhook
 
         return options
 
@@ -931,13 +949,6 @@ class TerminalInteractiveShell(InteractiveShell):
 
         return text
 
-    def enable_win_unicode_console(self):
-        # Since IPython 7.10 doesn't support python < 3.6 and PEP 528, Python uses the unicode APIs for the Windows
-        # console by default, so WUC shouldn't be needed.
-        warn("`enable_win_unicode_console` is deprecated since IPython 7.10, does not do anything and will be removed in the future",
-             DeprecationWarning,
-             stacklevel=2)
-
     def init_io(self):
         if sys.platform not in {'win32', 'cli'}:
             return
@@ -962,7 +973,7 @@ class TerminalInteractiveShell(InteractiveShell):
                 self.alias_manager.soft_define_alias(cmd, cmd)
 
     def __init__(self, *args, **kwargs) -> None:
-        super(TerminalInteractiveShell, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._set_autosuggestions(self.autosuggestions_provider)
         self.init_prompt_toolkit_cli()
         self.init_term_title()
@@ -1018,6 +1029,12 @@ class TerminalInteractiveShell(InteractiveShell):
 
     _inputhook = None
     def inputhook(self, context):
+        warn(
+            "inputkook seem unused, and marked for deprecation/Removal as of IPython 9.0. "
+            "Please open an issue if you are using it.",
+            category=PendingDeprecationWarning,
+            stacklevel=2,
+        )
         if self._inputhook is not None:
             self._inputhook(context)
 
